// Copyright (C) 2000 Jean-Marc Valin


#include "AudioProperties.h"
#include <iostream>
#include <string>
#include <strstream>

using namespace std;

static void set_sensitive (GnomePropertyBox *property_box, gint dirty)
{
   if (property_box->apply_button)
      gtk_widget_set_sensitive (property_box->apply_button, dirty);
}

static void gnome_property_box_unchanged (GnomePropertyBox *property_box)
{
   GtkWidget *page;
   
   g_return_if_fail (property_box != NULL);
   g_return_if_fail (GNOME_IS_PROPERTY_BOX (property_box));
   g_return_if_fail (property_box->notebook);
   g_return_if_fail (GTK_NOTEBOOK (property_box->notebook)->cur_page);
   
   page = GTK_NOTEBOOK (property_box->notebook)->cur_page->child;
   g_assert (page != NULL);
   
   gtk_object_set_data(GTK_OBJECT(page),
		       GNOME_PROPERTY_BOX_DIRTY,
		       GINT_TO_POINTER(0));
   
   set_sensitive (property_box, 0);
}


static void encoding_toggle (GtkToggleButton *button, AudioDataInfo::AudType type)
{
   if (gtk_toggle_button_get_active(button))
   {
      AudioProperties *prop = (AudioProperties *) gtk_object_get_data(GTK_OBJECT(button), "prop");
      prop->curr_info->encoding = type;
      prop->change();
      //cerr << "setting encoding to " << type << endl;
   }
}

static void stereo_toggle (GtkToggleButton *button, bool stereo)
{
   if (gtk_toggle_button_get_active(button))
   {
      AudioProperties *prop = (AudioProperties *) gtk_object_get_data(GTK_OBJECT(button), "prop");
      prop->curr_info->stereo = stereo;
      prop->change();
      //cerr << "setting mode to " << stereo << endl;
   }
}

static void sampling_select (GtkItem *item, char *sampl)
{
   GtkEntry *entr = (GtkEntry *) gtk_object_get_data(GTK_OBJECT(item), "entry");
   gtk_editable_delete_text(GTK_EDITABLE(entr), 0, -1);
   gtk_entry_append_text(entr, sampl);
}

static void prop_apply (GnomePropertyBox *propertybox, gint page, AudioProperties *prop)
{
   if (page != -1)
      return;
   prop->apply();
}

static void entry_changed (GtkEntry *entry, AudioProperties *prop)
{
   prop->change();
}

AudioProperties::AudioProperties()
   : info(new AudioDataInfo())
   , curr_info(new AudioDataInfo())
{
   setup();
}

AudioProperties::AudioProperties(const AudioProperties &proj_info)
   //: info(new AudioDataInfo())
   : info(new AudioDataInfo(*(proj_info.info)))
   , curr_info(new AudioDataInfo())
{
   setup();
}

AudioProperties::AudioProperties(xmlNodePtr prop)
   : info(new AudioDataInfo())
   , curr_info(new AudioDataInfo())
{
   info->sampling = atoi(((char *)xmlGetProp(prop, (CHAR *)"sampling")));
   info->frameSize = atoi(((char *)xmlGetProp(prop, (CHAR *)"framesize")));
   info->frameAdvance = atoi(((char *)xmlGetProp(prop, (CHAR *)"advance")));
   
   string enc((char *)xmlGetProp(prop, (CHAR *)"encoding"));
   if (enc == "ulaw")
      info->encoding = AudioDataInfo::ulaw;
   else if (enc == "alaw")
      info->encoding = AudioDataInfo::alaw;
   else if (enc == "lin8")
      info->encoding = AudioDataInfo::lin8;
   else if (enc == "lin16")
      info->encoding = AudioDataInfo::lin16;
   else if (enc == "float")
      info->encoding = AudioDataInfo::lin_float;
   else if (enc == "double")
      info->encoding = AudioDataInfo::lin_double;
   else 
      cerr << "what's that encoding: " << enc << endl;

   string stereo((char *)xmlGetProp(prop, (CHAR *)"stereo"));
   if (stereo == "true")
      info->stereo = true;
   else if (stereo == "false")
      info->stereo = false;
   else
      cerr << "stereo is neither true, nor false\n";
   setup();
}

void AudioProperties::setup()
{
   GtkWidget *notebook1;
   GtkWidget *hbox1;
   GtkWidget *aspectframe5;
   GtkWidget *vbox5;
   GtkWidget *entry3;
   GtkWidget *list1;
   GtkWidget *vbox1;
   GtkWidget *vbox2;
   GtkWidget *aspectframe2;
   GtkWidget *vbox4;
   GSList *vbox4_group = NULL;
   GtkWidget *radiobutton7;
   GtkWidget *radiobutton8;
   GtkWidget *aspectframe3;
   GtkWidget *table1;
   GtkWidget *label3;
   GtkWidget *label4;
   GtkWidget *entry1;
   GtkWidget *entry2;
   GtkWidget *aspectframe4;
   GtkWidget *vbox3;
   GSList *vbox3_group = NULL;
   GtkWidget *radiobutton1;
   GtkWidget *radiobutton2;
   GtkWidget *radiobutton3;
   GtkWidget *radiobutton5;
   GtkWidget *radiobutton4;
   GtkWidget *radiobutton6;
   GtkWidget *label1;
   GtkWidget *empty_notebook_page;
   GtkWidget *label2;
   GtkWidget *list_item;
   GList *items;
   
   dialog = gnome_property_box_new ();
   gnome_dialog_close_hides (GNOME_DIALOG(dialog), TRUE);
   gtk_signal_connect (GTK_OBJECT (dialog), "apply",
		       GTK_SIGNAL_FUNC(prop_apply), this);
   gtk_object_set_data (GTK_OBJECT (dialog), "dialog", dialog);
   
   notebook1 = GNOME_PROPERTY_BOX (dialog)->notebook;
   gtk_object_set_data (GTK_OBJECT (dialog), "notebook1", notebook1);
   gtk_widget_show (notebook1);
   
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_widget_ref (hbox1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "hbox1", hbox1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (hbox1);
   gtk_container_add (GTK_CONTAINER (notebook1), hbox1);
   
   aspectframe5 = gtk_aspect_frame_new (_("Sampling Rate"), 0.5, 0, 1, TRUE);
   gtk_widget_ref (aspectframe5);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "aspectframe5", aspectframe5,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (aspectframe5);
   gtk_box_pack_start (GTK_BOX (hbox1), aspectframe5, TRUE, TRUE, 0);
   
   vbox5 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox5);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox5", vbox5,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (vbox5);
   gtk_container_add (GTK_CONTAINER (aspectframe5), vbox5);
   
   entry3 = gtk_entry_new ();
   gtk_widget_ref (entry3);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "entry3", entry3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (entry3);
   gtk_box_pack_start (GTK_BOX (vbox5), entry3, FALSE, FALSE, 0);
   sampling_entr = GTK_ENTRY(entry3);
   
   gtk_signal_connect (GTK_OBJECT (entry3), "changed",
		       GTK_SIGNAL_FUNC(entry_changed), this);
   
   
   list1 = gtk_list_new ();
   gtk_widget_ref (list1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list1", list1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (list1);
   gtk_box_pack_start (GTK_BOX (vbox5), list1, TRUE, TRUE, 0);
   
   
   list_item = gtk_list_item_new_with_label("8000");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item1", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"8000");
   gtk_widget_show(list_item);
   items = g_list_append(NULL, list_item);
   
   list_item = gtk_list_item_new_with_label("12000");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item2", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"12000");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   list_item = gtk_list_item_new_with_label("16000");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item3", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"16000");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   list_item = gtk_list_item_new_with_label("22050");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item4", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"22050");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   list_item = gtk_list_item_new_with_label("32000");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item5", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"32000");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   list_item = gtk_list_item_new_with_label("44100");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item6", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"44100");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   list_item = gtk_list_item_new_with_label("48000");
   gtk_widget_ref (list_item);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "list_item", list_item,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (list_item), "entry", entry3);  
   gtk_signal_connect (GTK_OBJECT (list_item), "select",
		       GTK_SIGNAL_FUNC (sampling_select),
		       (void *)"48000");
   gtk_widget_show(list_item);
   items = g_list_append(items, list_item);
   
   gtk_list_append_items(GTK_LIST(list1), items);
   
   
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox1", vbox1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (vbox1);
   gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
   
   vbox2 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox2);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox2", vbox2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (vbox2);
   gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);
   
   aspectframe2 = gtk_aspect_frame_new (_("Mode"), 0.5, 0, 1, TRUE);
   gtk_widget_ref (aspectframe2);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "aspectframe2", aspectframe2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (aspectframe2);
   gtk_box_pack_start (GTK_BOX (vbox2), aspectframe2, TRUE, TRUE, 0);
   
   vbox4 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox4);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox4", vbox4,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (vbox4);
   gtk_container_add (GTK_CONTAINER (aspectframe2), vbox4);
   
   radiobutton7 = gtk_radio_button_new_with_label (vbox4_group, _("mono"));
   vbox4_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton7));
   gtk_widget_ref (radiobutton7);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton7", radiobutton7,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton7), "prop", this);
   gtk_widget_show (radiobutton7);
   gtk_box_pack_start (GTK_BOX (vbox4), radiobutton7, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton7), "toggled",
		       GTK_SIGNAL_FUNC (stereo_toggle),
		       (void *)0);
   stereo_buttons[0] = GTK_TOGGLE_BUTTON (radiobutton7);
   
   radiobutton8 = gtk_radio_button_new_with_label (vbox4_group, _("stereo"));
   vbox4_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton8));
   gtk_widget_ref (radiobutton8);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton8", radiobutton8,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton8), "prop", this);
   gtk_widget_show (radiobutton8);
   gtk_box_pack_start (GTK_BOX (vbox4), radiobutton8, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton8), "toggled",
		       GTK_SIGNAL_FUNC (stereo_toggle),
		       (void *)1);
   stereo_buttons[1] = GTK_TOGGLE_BUTTON (radiobutton8);
   
   aspectframe3 = gtk_aspect_frame_new (_("Frames"), 0.5, 0.5, 1, TRUE);
   gtk_widget_ref (aspectframe3);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "aspectframe3", aspectframe3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (aspectframe3);
   gtk_box_pack_start (GTK_BOX (vbox2), aspectframe3, TRUE, TRUE, 0);
   
   table1 = gtk_table_new (2, 2, FALSE);
   gtk_widget_ref (table1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "table1", table1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (table1);
   gtk_container_add (GTK_CONTAINER (aspectframe3), table1);
   
   label3 = gtk_label_new (_("Frame Size"));
   gtk_widget_ref (label3);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "label3", label3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label3);
   gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 0, 1,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   label4 = gtk_label_new (_("Subframe Size"));
   gtk_widget_ref (label4);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "label4", label4,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label4);
   gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 1, 2,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   entry1 = gtk_entry_new ();
   gtk_widget_ref (entry1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "entry1", entry1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (entry1);
   gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   frame_size_entr = GTK_ENTRY(entry1);
   gtk_signal_connect (GTK_OBJECT (entry1), "changed",
		       GTK_SIGNAL_FUNC(entry_changed), this);
   
   entry2 = gtk_entry_new ();
   gtk_widget_ref (entry2);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "entry2", entry2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (entry2);
   gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   frame_advance_entr = GTK_ENTRY(entry2);
   gtk_signal_connect (GTK_OBJECT (entry2), "changed",
		       GTK_SIGNAL_FUNC(entry_changed), this);
   
   aspectframe4 = gtk_aspect_frame_new (_("Encoding"), 0.5, 0, 1, TRUE);
   gtk_widget_ref (aspectframe4);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "aspectframe4", aspectframe4,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (aspectframe4);
   gtk_box_pack_start (GTK_BOX (hbox1), aspectframe4, TRUE, TRUE, 0);
   
   vbox3 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox3);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "vbox3", vbox3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (vbox3);
   gtk_container_add (GTK_CONTAINER (aspectframe4), vbox3);
   
   radiobutton1 = gtk_radio_button_new_with_label (vbox3_group, _("U-Law"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
   gtk_widget_ref (radiobutton1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton1", radiobutton1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton1), "prop", this);
   gtk_widget_show (radiobutton1);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton1, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::ulaw);
   enc_buttons[AudioDataInfo::ulaw] = GTK_TOGGLE_BUTTON (radiobutton1);
   
   radiobutton2 = gtk_radio_button_new_with_label (vbox3_group, _("A-Law"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
   gtk_widget_ref (radiobutton2);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton2", radiobutton2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton2), "prop", this);
   gtk_widget_show (radiobutton2);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton2, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton2), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::alaw);
   enc_buttons[AudioDataInfo::alaw] = GTK_TOGGLE_BUTTON (radiobutton2);
   
   radiobutton3 = gtk_radio_button_new_with_label (vbox3_group, _("char (linear 8)"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton3));
   gtk_widget_ref (radiobutton3);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton3", radiobutton3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton3), "prop", this);
   gtk_widget_show (radiobutton3);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton3, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton3), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::lin8);
   enc_buttons[AudioDataInfo::lin8] = GTK_TOGGLE_BUTTON (radiobutton3);
   
   radiobutton5 = gtk_radio_button_new_with_label (vbox3_group, _("short (linear 16)"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton5));
   gtk_widget_ref (radiobutton5);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton5", radiobutton5,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton5), "prop", this);
   gtk_widget_show (radiobutton5);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton5, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton5), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::lin16);
   enc_buttons[AudioDataInfo::lin16] = GTK_TOGGLE_BUTTON (radiobutton5);
   
   radiobutton4 = gtk_radio_button_new_with_label (vbox3_group, _("float"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton4));
   gtk_widget_ref (radiobutton4);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton4", radiobutton4,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton4), "prop", this);
   gtk_widget_show (radiobutton4);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton4, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton4), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::lin_float);
   enc_buttons[AudioDataInfo::lin_float] = GTK_TOGGLE_BUTTON (radiobutton4);
   
   radiobutton6 = gtk_radio_button_new_with_label (vbox3_group, _("double"));
   vbox3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton6));
   gtk_widget_ref (radiobutton6);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "radiobutton6", radiobutton6,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data (GTK_OBJECT (radiobutton6), "prop", this);
   gtk_widget_show (radiobutton6);
   gtk_box_pack_start (GTK_BOX (vbox3), radiobutton6, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (radiobutton6), "toggled",
		       GTK_SIGNAL_FUNC (encoding_toggle),
		       (void *)AudioDataInfo::lin_double);
   enc_buttons[AudioDataInfo::lin_double] = GTK_TOGGLE_BUTTON (radiobutton6);
   
   label1 = gtk_label_new (_("Audio"));
   gtk_widget_ref (label1);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "label1", label1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label1);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);
   
   empty_notebook_page = gtk_vbox_new (FALSE, 0);
   gtk_widget_show (empty_notebook_page);
   gtk_container_add (GTK_CONTAINER (notebook1), empty_notebook_page);
   
   label2 = gtk_label_new (_("Processing"));
   gtk_widget_ref (label2);
   gtk_object_set_data_full (GTK_OBJECT (dialog), "label2", label2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label2);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);
   
}

AudioProperties::~AudioProperties()
{
   gtk_widget_destroy(dialog);
   delete info;
   delete curr_info;
}

void AudioProperties::show()
{
   *curr_info = *info;
   gtk_toggle_button_set_active (enc_buttons[info->encoding], true);
   gtk_toggle_button_set_active (stereo_buttons[info->stereo ? 1 : 0], true);
   
   char ch[11];
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->sampling;
      gtk_entry_set_text (sampling_entr, ch);
   }
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->frameSize;
      gtk_entry_set_text (frame_size_entr, ch);
   }
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->frameAdvance;
      gtk_entry_set_text (frame_advance_entr, ch);
   }
   gnome_property_box_unchanged(GNOME_PROPERTY_BOX(dialog));
   gtk_widget_show(dialog);
}

void AudioProperties::apply()
{
   curr_info->sampling = atoi(gtk_entry_get_text (sampling_entr));
   curr_info->frameSize = atoi(gtk_entry_get_text (frame_size_entr));
   curr_info->frameAdvance = atoi(gtk_entry_get_text (frame_advance_entr));
   *info = *curr_info;
}

void AudioProperties::change()
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(dialog));
}

void AudioProperties::save (xmlNode *root)
{
   xmlNodePtr tree;
   tree = xmlNewChild(root, NULL, (CHAR *)"Properties", NULL);
      char ch[11];
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->sampling;
      xmlSetProp(tree, (CHAR *)"sampling", (CHAR *)ch);
   }
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->frameSize;
      xmlSetProp(tree, (CHAR *)"framesize", (CHAR *)ch);
   }
   {
      strstream str(ch, 10);
      for (int i=0;i<11;i++) ch[i]=0;
      str << info->frameAdvance;
      xmlSetProp(tree, (CHAR *)"advance", (CHAR *)ch);
   }

   if (info->stereo)
      xmlSetProp(tree, (CHAR *)"stereo", (CHAR *)"true");
   else
      xmlSetProp(tree, (CHAR *)"stereo", (CHAR *)"false");
   switch (info->encoding)
   {
      case AudioDataInfo::ulaw:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"ulaw");
	 break;
      case AudioDataInfo::alaw:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"alaw");
	 break;
      case AudioDataInfo::lin8:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"lin8");
	 break;
      case AudioDataInfo::lin16:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"lin16");
	 break;
      case AudioDataInfo::lin_float:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"float");
	 break;
      case AudioDataInfo::lin_double:
	 xmlSetProp(tree, (CHAR *)"encoding", (CHAR *)"double");
	 break;

   }
 
}
