// Copyright (C) 1999 Jean-Marc Valin


#include "VisixFileSource.h"
#include "AudioDataFile.h"
#include "VisixProject.h"

VisixFileSource::VisixFileSource(GtkWidget *_parent, VisixProject *_project, const string &_name, const string &_path)
   : VisixSource(_parent, _project, _name)
   , path(_path)
{
   properties = new AudioProperties(*(_project->getProperties()));
}

VisixFileSource::VisixFileSource(GtkWidget *_parent, VisixProject *_project, xmlNodePtr src)
   : VisixSource(_parent, _project, string((char *)xmlGetProp(src, (CHAR *)"name")))
   , path(string((char *)xmlGetProp(src, (CHAR *)"path")))
{
   xmlNodePtr prop = src->childs;
   bool found = false;
   //cerr << "searching for properties\n";
   while (prop != NULL)
   {
      if (string((char*)prop->name) == "Properties")
      {
	 found = true;
	 properties = new AudioProperties(prop);
	 break;
      }
      prop = prop->next;
   }
   if (!found)
   {
      properties = new AudioProperties(*(_project->getProperties()));
   }
   string select((char *)xmlGetProp(src, (CHAR *)"selected"));
   selected = false;;
   if (select == "true")
   {
      toggle();
   }
}

void VisixFileSource::loadSource()
{
   //cerr << "loading...\n";
   source = new AudioDataFile (path+name, properties->getInfo());
}

void VisixFileSource::unloadSource()
{
   //cerr << "unloading...\n";
   delete source;
   source = NULL;
}

void VisixFileSource::save (xmlNode *root)
{
   xmlNodePtr tree;
   tree = xmlNewChild(root, NULL, (CHAR *)"FileSource", NULL);
   xmlSetProp(tree, (CHAR *)"name", (CHAR *)name.c_str());
   xmlSetProp(tree, (CHAR *)"path", (CHAR *)path.c_str());
   xmlSetProp(tree, (CHAR *)"selected", (xmlChar *) (selected ? "true" : "false") );
   properties->save(tree);
 
}
